﻿Imports System.Windows.Navigation
Imports Microsoft.Phone.Controls
Imports Microsoft.Phone.Shell
Imports Microsoft.Xna.Framework
Imports Microsoft.Xna.Framework.Content
Imports Microsoft.Xna.Framework.Graphics

Public Enum StartState
    FreshStart
    TombstoneResurrected
    NonTombstoneReactivated
End Enum

Partial Public Class App
    Inherits Application

#Region "Fields/Properties"

    Private phoneApplicationInitialized As Boolean = False
    Private _rootFrame As PhoneApplicationFrame
    Public Property RootFrame() As PhoneApplicationFrame
        Get
            Return _rootFrame
        End Get
        Private Set(ByVal value As PhoneApplicationFrame)
            _rootFrame = value
        End Set
    End Property
    Public Shared StartState As StartState
    Private wasTerminated As Boolean = True


    Private Shared _viewModel As MainViewModel = Nothing
    Public Shared ReadOnly Property ViewModel() As MainViewModel
        Get
            ' Delay creation of the view model until necessary.
            If _viewModel Is Nothing Then
                _viewModel = New MainViewModel()
            End If
            Return _viewModel
        End Get
    End Property

    ' Additional properties for XNA integration.
    Private _content As ContentManager
    Public Property Content() As ContentManager
        Get
            Return _content
        End Get
        Private Set(ByVal value As ContentManager)
            _content = value
        End Set
    End Property
    Private _services As AppServiceProvider
    Public Property Services() As AppServiceProvider
        Get
            Return _services
        End Get
        Private Set(ByVal value As AppServiceProvider)
            _services = value
        End Set
    End Property

    ' Provides access to a GameTimer that is set up to pump the FrameworkDispatcher.
    Private _frameworkDispatcherTimer As GameTimer
    Public Property FrameworkDispatcherTimer() As GameTimer
        Get
            Return _frameworkDispatcherTimer
        End Get
        Private Set(ByVal value As GameTimer)
            _frameworkDispatcherTimer = value
        End Set
    End Property

    Private isXnaInitialized As Boolean

#End Region


#Region "Init"

    Public Sub New()
        ' Global handler for uncaught exceptions. 
        AddHandler UnhandledException, AddressOf Application_UnhandledException

        ' Show graphics profiling information while debugging.
        'If System.Diagnostics.Debugger.IsAttached Then
        'Application.Current.Host.Settings.EnableFrameRateCounter = True
        'Application.Current.Host.Settings.EnableRedrawRegions = True
        ' Application.Current.Host.Settings.EnableCacheVisualization = True

        ' Disable the application idle detection by setting the UserIdleDetectionMode property of the
        ' application's PhoneApplicationService object to Disabled.
        ' Caution:- Use this under debug mode only. Application that disables user idle detection will continue to run
        ' and consume battery power when the user is not using the phone.
        'PhoneApplicationService.Current.UserIdleDetectionMode = IdleDetectionMode.Disabled;
        ' End If

        ' Standard Silverlight initialization.
        InitializeComponent()

        ' Phone-specific initialization.
        InitializePhoneApplication()

        ' UNDONE: Defer XNA initialization until the user navigates to the Game page -
        ' this will improve startup time.
        ' Added for XNA integration.
        'InitializeXnaApplication();
    End Sub

    ' Do not add any additional code to this method
    Private Sub InitializePhoneApplication()
        If phoneApplicationInitialized Then
            Return
        End If

        ' Create the frame but don't set it as RootVisual yet; this allows the splash
        ' screen to remain active until the application is ready to render.
        ' Use the Toolkit TransitionFrame so that we can support page transition animations.
        'RootFrame = new PhoneApplicationFrame();
        RootFrame = New TransitionFrame()
        AddHandler RootFrame.Navigated, AddressOf CompleteInitializePhoneApplication

        ' Handle navigation failures
        AddHandler RootFrame.NavigationFailed, AddressOf RootFrame_NavigationFailed

        ' Ensure we don't initialize again
        phoneApplicationInitialized = True
    End Sub

    ' Do not add any additional code to this method
    Private Sub CompleteInitializePhoneApplication(ByVal sender As Object, ByVal e As NavigationEventArgs)
        ' Set the root visual to allow the application to render
        If RootVisual IsNot RootFrame Then
            RootVisual = RootFrame
        End If

        ' Remove this handler since it is no longer needed
        RemoveHandler RootFrame.Navigated, AddressOf CompleteInitializePhoneApplication
    End Sub

#End Region


#Region "Lifetime events"

    ' Code to execute when the application is launching (eg, from Start)
    ' This code will not execute when the application is reactivated
    Private Sub Application_Launching(ByVal sender As Object, ByVal e As LaunchingEventArgs)
        ' This is the fresh start case:
        ' [Closing-->]Ctor-->Launching
        StartState = StartState.FreshStart

        ViewModel.LoadData()
    End Sub

    ' Code to execute when the application is activated (brought to foreground)
    ' This code will not execute when the application is first launched
    Private Sub Application_Activated(ByVal sender As Object, ByVal e As ActivatedEventArgs)
        If wasTerminated Then
            ' This is the tombstone/resurrection case:
            ' Deactivated-->Ctor-->Activated
            StartState = StartState.TombstoneResurrected
        Else
            ' The non-tombstone case:
            ' Deactivated-->Activated
            StartState = StartState.NonTombstoneReactivated
        End If

        ' Ensure that application state is restored appropriately
        If Not App.ViewModel.IsDataLoaded Then
            App.ViewModel.LoadData()
        End If
    End Sub

    ' Code to execute when the application is deactivated (sent to background)
    ' This code will not execute when the application is closing
    Private Sub Application_Deactivated(ByVal sender As Object, ByVal e As DeactivatedEventArgs)
        ' Ensure that required application state is persisted here.
        wasTerminated = False
    End Sub

    ' Code to execute when the application is closing (eg, user hit Back)
    ' This code will not execute when the application is deactivated
    Private Sub Application_Closing(ByVal sender As Object, ByVal e As ClosingEventArgs)
    End Sub

#End Region


#Region "Exceptions"

    ' Code to execute if a navigation fails
    Private Sub RootFrame_NavigationFailed(ByVal sender As Object, ByVal e As NavigationFailedEventArgs)
        If System.Diagnostics.Debugger.IsAttached Then
            ' A navigation has failed; break into the debugger
            System.Diagnostics.Debugger.Break()
        Else
            If e.Exception IsNot Nothing AndAlso (Not String.IsNullOrEmpty(e.Exception.Message)) Then
                MessageBox.Show(String.Format("Sorry, we've encountered an error: {0}.", e.Exception.Message))
            Else
                MessageBox.Show("Sorry, we've encountered an error.")
            End If
            e.Handled = True
        End If
    End Sub

    ' Code to execute on Unhandled Exceptions
    Private Sub Application_UnhandledException(ByVal sender As Object, ByVal e As ApplicationUnhandledExceptionEventArgs)
        If System.Diagnostics.Debugger.IsAttached Then
            ' An unhandled exception has occurred; break into the debugger
            System.Diagnostics.Debugger.Break()
        Else
            If e.ExceptionObject IsNot Nothing AndAlso (Not String.IsNullOrEmpty(e.ExceptionObject.Message)) Then
                MessageBox.Show(String.Format("Sorry, we've encountered an error: {0}.", e.ExceptionObject.Message))
            Else
                MessageBox.Show("Sorry, we've encountered an error.")
            End If
            e.Handled = True
        End If
    End Sub

#End Region


#Region "XNA application initialization"

    ' Added for XNA integration.
    ' Performs initialization of the XNA types required for the application.
    Friend Sub InitializeXnaApplication()
        If Not isXnaInitialized Then
            ' Create the service provider
            Services = New AppServiceProvider()

            ' Add the SharedGraphicsDeviceManager to the Services as the IGraphicsDeviceService for the app
            For Each obj As Object In ApplicationLifetimeObjects
                If TypeOf obj Is IGraphicsDeviceService Then
                    Services.AddService(GetType(IGraphicsDeviceService), obj)
                End If
            Next obj

            ' Create the ContentManager so the application can load precompiled assets
            Content = New ContentManager(Services, "Content")

            ' Create a GameTimer to pump the XNA FrameworkDispatcher
            FrameworkDispatcherTimer = New GameTimer()
            AddHandler FrameworkDispatcherTimer.FrameAction, AddressOf FrameworkDispatcherFrameAction
            FrameworkDispatcherTimer.Start()

            isXnaInitialized = True
        End If
    End Sub

    ' An event handler that pumps the FrameworkDispatcher each frame.
    ' FrameworkDispatcher is required for a lot of the XNA events and
    ' for certain functionality such as SoundEffect playback.
    Private Sub FrameworkDispatcherFrameAction(ByVal sender As Object, ByVal e As EventArgs)
        FrameworkDispatcher.Update()
    End Sub

#End Region

End Class
