﻿' Animation.vb
Imports Microsoft.VisualBasic
Imports Microsoft.Xna.Framework
Imports Microsoft.Xna.Framework.Content
Imports Microsoft.Xna.Framework.Graphics

Public Class Animation
    ' The image representing the collection of images used for animation
    Private spriteStrip As Texture2D

    ' The scale used to display the sprite strip
    Private scale As Single

    ' The time since we last updated the frame
    Private elapsed As Integer

    ' The time we display a frame until the next one
    Private frameTime As Integer

    ' The number of frames that the animation contains
    Private frameCount As Integer

    ' The index of the current frame we are displaying
    Private currentFrame As Integer

    ' The color of the frame we will be displaying
    Private color As Color

    ' The area of the image strip we want to display
    Private sourceRect As New Rectangle()

    ' The area where we want to display the image strip in the game
    Private destinationRect As New Rectangle()

    ' Width of a given frame
    Public FrameWidth As Integer

    ' Height of a given frame
    Public FrameHeight As Integer

    ' The state of the Animation
    Public Active As Boolean

    ' Determines if the animation will keep playing or deactivate after one run
    Public Looping As Boolean

    ' Width of a given frame
    Public Position As Vector2

    Public Sub Initialize(ByVal texture As Texture2D, ByVal position As Vector2, ByVal frameWidth As Integer, ByVal frameHeight As Integer, ByVal frameCount As Integer, ByVal frametime As Integer, ByVal color As Color, ByVal scale As Single, ByVal looping As Boolean)
        ' Keep a local copy of the values passed in
        Me.color = color
        Me.FrameWidth = frameWidth
        Me.FrameHeight = frameHeight
        Me.frameCount = frameCount
        Me.frameTime = frametime
        Me.scale = scale

        Me.Looping = looping
        Me.Position = position
        spriteStrip = texture

        ' Set the time to zero
        elapsed = 0
        currentFrame = 0

        ' Set the Animation to active by default
        Active = True
    End Sub

    Public Sub Update(ByVal totalTime As TimeSpan, ByVal elapsedTime As TimeSpan)
        ' Do not update the game if we are not active
        If Active = False Then
            Return
        End If

        ' Update the elapsed time
        elapsed += CInt(Fix(elapsedTime.TotalMilliseconds))

        ' If the elapsed time is larger than the frame time
        ' we need to switch frames
        If elapsed > frameTime Then
            ' Move to the next frame
            currentFrame += 1

            ' If the currentFrame is equal to frameCount reset currentFrame to zero
            If currentFrame = frameCount Then
                currentFrame = 0
                ' If we are not looping deactivate the animation
                If Looping = False Then
                    Active = False
                End If
            End If

            ' Reset the elapsed time to zero
            elapsed = 0
        End If

        ' Grab the correct frame in the image strip by multiplying the currentFrame index by the frame width
        sourceRect = New Rectangle(currentFrame * FrameWidth, 0, FrameWidth, FrameHeight)

        ' Grab the correct frame in the image strip by multiplying the currentFrame index by the frame width
        destinationRect = New Rectangle(CInt(Fix(Position.X)) - CInt(Fix(FrameWidth * scale)) / 2, CInt(Fix(Position.Y)) - CInt(Fix(FrameHeight * scale)) / 2, CInt(Fix(FrameWidth * scale)), CInt(Fix(FrameHeight * scale)))
    End Sub

    ' Draw the Animation Strip
    Public Sub Draw(ByVal spriteBatch As SpriteBatch)
        ' Only draw the animation when we are active
        If Active Then
            spriteBatch.Draw(spriteStrip, destinationRect, sourceRect, color)
        End If
    End Sub
End Class