﻿Imports Microsoft.Xna.Framework
Imports Microsoft.Xna.Framework.Graphics

Friend Class Enemy
    ' Animation representing the enemy
    Public EnemyAnimation As Animation

    ' The position of the enemy ship relative to the top left corner of thescreen
    Public Position As Vector2

    ' The state of the Enemy Ship
    Public Active As Boolean

    ' The hit points of the enemy, if this goes to zero the enemy dies
    Public Health As Integer

    ' The amount of damage the enemy inflicts on the player ship
    Public Damage As Integer

    ' The amount of score the enemy will give to the player
    Public Value As Integer

    ' Get the width of the enemy ship
    Public ReadOnly Property Width() As Integer
        Get
            Return EnemyAnimation.FrameWidth
        End Get
    End Property

    ' Get the height of the enemy ship
    Public ReadOnly Property Height() As Integer
        Get
            Return EnemyAnimation.FrameHeight
        End Get
    End Property

    ' The speed at which the enemy moves
    Private enemyMoveSpeed As Single

    Public Sub Initialize(ByVal animation As Animation, ByVal position As Vector2)
        ' Load the enemy ship texture
        EnemyAnimation = animation

        ' Set the position of the enemy
        Me.Position = position

        ' We initialize the enemy to be active so it will be updated
        Active = True

        ' Set the health of the enemy
        Health = 10

        ' Set the amount of damage the enemy can do
        Damage = 10

        ' Set how fast the enemy moves
        enemyMoveSpeed = 6.0F

        ' Set the score value of the enemy
        Value = 100
    End Sub

    Public Sub Update(ByVal totalTime As TimeSpan, ByVal elapsedTime As TimeSpan)
        ' The enemy always moves to the left so decrement x
        Position.X -= enemyMoveSpeed

        ' Update the position of the Animation
        EnemyAnimation.Position = Position

        ' Update Animation
        EnemyAnimation.Update(totalTime, elapsedTime)

        ' If the enemy is past the screen or its health reaches 0, deactivate
        If Position.X < -Width OrElse Health <= 0 Then
            ' By setting the Active flag to false, the game will remove
            Active = False
        End If
    End Sub

    Public Sub Draw(ByVal spriteBatch As SpriteBatch)
        ' Draw the animation
        EnemyAnimation.Draw(spriteBatch)
    End Sub
End Class