﻿Imports Microsoft.Xna.Framework
Imports Microsoft.Xna.Framework.Content
Imports Microsoft.Xna.Framework.Graphics

Public Class ParallaxingBackground
    ' The image representing the parallaxing background.
    Private texture As Texture2D

    ' An array of positions of the parallaxing background.
    Private positions() As Vector2

    ' The speed which the background is moving.
    Private speed As Integer

    Public Sub Initialize(ByVal content As ContentManager, ByVal texturePath As String, ByVal screenWidth As Integer, ByVal speed As Integer)
        ' Load the background texture we will be using.
        texture = content.Load(Of Texture2D)(texturePath)

        ' Set the speed of the background.
        Me.speed = speed

        ' If we divide the screen with the texture width then we can determine the number of tiles needed.
        ' We add 1 to it so that we won't have a gap in the tiling.
        positions = New Vector2(screenWidth \ texture.Width) {}

        ' Set the initial positions of the parallaxing background.
        For i = 0 To positions.Length - 1
            ' We need the tiles to be side by side to create a tiling effect.
            positions(i) = New Vector2(i * texture.Width, 0)
        Next i
    End Sub

    Public Sub Update()
        ' Update the positions of the background.
        For i = 0 To positions.Length - 1
            ' Update the position of the screen by adding the speed.
            positions(i).X += speed

            ' If the speed has the background moving to the left.
            If speed <= 0 Then
                ' Check the texture is out of view then put that texture at the end of the screen,
                If positions(i).X <= -texture.Width Then
                    positions(i).X = texture.Width * (positions.Length - 1)
                End If

                ' If the speed has the background moving to the right,
            Else
                ' Check if the texture is out of view then position it to the start of the screen,
                If positions(i).X >= texture.Width * (positions.Length - 1) Then
                    positions(i).X = -texture.Width
                End If
            End If
        Next i
    End Sub

    Public Sub Draw(ByVal spriteBatch As SpriteBatch)
        For i = 0 To positions.Length - 1
            spriteBatch.Draw(texture, positions(i), Color.White)
        Next i
    End Sub
End Class
