﻿Imports Microsoft.Xna.Framework
Imports Microsoft.Xna.Framework.Graphics

Friend Class Projectile
    ' Image representing the Projectile.
    Public Texture As Texture2D

    ' Position of the Projectile relative to the upper left side of the screen.
    Public Position As Vector2

    Public Active As Boolean

    ' The amount of damage the projectile can inflict to an enemy.
    Public Damage As Integer

    ' Represents the viewable boundary of the game.
    Private viewport As Viewport

    Public ReadOnly Property Width() As Integer
        Get
            Return Texture.Width
        End Get
    End Property

    Public ReadOnly Property Height() As Integer
        Get
            Return Texture.Height
        End Get
    End Property

    ' Determines how fast the projectile moves.
    Private projectileMoveSpeed As Single

    Public Sub Initialize(ByVal viewport As Viewport, ByVal texture As Texture2D, ByVal position As Vector2)
        Me.Texture = texture
        Me.Position = position
        Me.viewport = viewport
        Active = True
        Damage = 2
        projectileMoveSpeed = 20.0F
    End Sub

    Public Sub Update()
        ' Projectiles always move to the right.
        Position.X += projectileMoveSpeed

        ' Deactivate the bullet if it goes out of screen.
        If Position.X + Texture.Width \ 2 > viewport.Width Then
            Active = False
        End If
    End Sub

    Public Sub Draw(ByVal spriteBatch As SpriteBatch)
        spriteBatch.Draw(Texture, Position, Nothing, Color.White, 0.0F, New Vector2(Width \ 2, Height \ 2), 1.0F, SpriteEffects.None, 0.0F)
    End Sub
End Class