﻿Imports System.Windows.Navigation
Imports Microsoft.Phone.Controls
Imports Microsoft.Xna.Framework
Imports Microsoft.Xna.Framework.Audio
Imports Microsoft.Xna.Framework.Content
Imports Microsoft.Xna.Framework.Graphics
Imports Microsoft.Xna.Framework.Input.Touch
Imports System.IO.IsolatedStorage

Partial Public Class GamePage
    Inherits PhoneApplicationPage

#Region "Properties"

    Public Property GraphicsDevice() As GraphicsDevice

#End Region


#Region "Fields"

    Private contentManager As ContentManager
    Private timer As GameTimer
    Private spriteBatch As SpriteBatch
    Private mainBackground As Texture2D
    Private backgroundLayer1 As ParallaxingBackground
    Private backgroundLayer2 As ParallaxingBackground
    Private enemyTexture As Texture2D
    Private enemies As List(Of Enemy)
    Private enemySpawnTime As TimeSpan
    Private previousSpawnTime As TimeSpan
    Private random As Random
    Private projectileTexture As Texture2D
    Private projectiles As List(Of Projectile)
    Private fireTime As TimeSpan
    Private previousFireTime As TimeSpan
    Private explosionTexture As Texture2D
    Private explosions As List(Of Animation)
    Private font As SpriteFont
    Private projectileSound As SoundEffect
    Private collisionSound As SoundEffect
    Private warningSound As SoundEffect
    Private gameOverSound As SoundEffect
    Private warningSoundInstance As SoundEffectInstance
    Private score As Integer
    Private player As Player
    Private xBounds As Integer
    Private yBounds As Integer
    Private isPlayerAlive As Boolean
    Private initialPlayerPosition As Vector2
    Private isContentLoaded As Boolean
    Private playerAnimation As Animation

#End Region


#Region "Init"

    Public Sub New()
        InitializeComponent()

        CType(App.Current, MangoClient.App).InitializeXnaApplication()

        ' Create a timer for this page
        timer = New GameTimer()
        timer.UpdateInterval = TimeSpan.FromTicks(333333)
        AddHandler timer.Update, AddressOf OnUpdate
        AddHandler timer.Draw, AddressOf OnDraw

        ' Get a private reference to the appp's ContentManager.
        contentManager = (TryCast(Application.Current, App)).Content
        GraphicsDevice = SharedGraphicsDeviceManager.Current.GraphicsDevice

        ' Enable FreeDrag.
        TouchPanel.EnabledGestures = GestureType.FreeDrag Or GestureType.Tap

        ' Initialize the player, background, and enemies and projectiles collections.
        player = New Player()
        backgroundLayer1 = New ParallaxingBackground()
        backgroundLayer2 = New ParallaxingBackground()
        enemies = New List(Of Enemy)()
        previousSpawnTime = TimeSpan.Zero
        enemySpawnTime = TimeSpan.FromSeconds(1.0F)
        random = New Random()
        projectiles = New List(Of Projectile)()
        explosions = New List(Of Animation)()
        score = 0
        player.Health = 100
        fireTime = TimeSpan.FromSeconds(0.15F)
        isPlayerAlive = True

        ' Set the starting position of the player around the middle of the screen and to the back.
        initialPlayerPosition = New Vector2(GraphicsDevice.Viewport.TitleSafeArea.X, GraphicsDevice.Viewport.TitleSafeArea.Y + GraphicsDevice.Viewport.TitleSafeArea.Height \ 2)
    End Sub

#End Region


#Region "LoadContent"

    Public Sub LoadContent()
        ' Load the image and sound content.
        backgroundLayer1.Initialize(contentManager, "backgroundLayer1", GraphicsDevice.Viewport.Height, -1)
        backgroundLayer2.Initialize(contentManager, "backgroundLayer2", GraphicsDevice.Viewport.Height, -2)
        mainBackground = contentManager.Load(Of Texture2D)("mainbackground")
        enemyTexture = contentManager.Load(Of Texture2D)("mineAnimation")
        projectileTexture = contentManager.Load(Of Texture2D)("laser")
        explosionTexture = contentManager.Load(Of Texture2D)("explosion")
        projectileSound = contentManager.Load(Of SoundEffect)("sound/projectile")
        collisionSound = contentManager.Load(Of SoundEffect)("sound/collision")
        warningSound = contentManager.Load(Of SoundEffect)("sound/warning")
        gameOverSound = contentManager.Load(Of SoundEffect)("sound/gameover")
        font = contentManager.Load(Of SpriteFont)("gameFont")
        warningSoundInstance = warningSound.CreateInstance()
        warningSoundInstance.IsLooped = True

        ' Create a new SpriteBatch, which can be used to draw textures.
        spriteBatch = New SpriteBatch(GraphicsDevice)

        ' Load the player resources.
        playerAnimation = New Animation()
        Dim playerTexture As Texture2D = contentManager.Load(Of Texture2D)("shipAnimation")

        ' Each frame in the animation strip is 150 wide, the strip is 75 high, and there are 5 frames.
        playerAnimation.Initialize(playerTexture, Vector2.Zero, 150, 75, 5, 30, Color.White, 1.0F, True)

        isContentLoaded = True
    End Sub

    Private Sub InitializePlayer(ByVal newPosition As Vector2)
        player.Initialize(playerAnimation, newPosition)
        xBounds = player.Width \ 2
        yBounds = player.Height \ 2
    End Sub

#End Region


#Region "Manage Explosions"
    Private Sub AddExplosion(ByVal position As Vector2)
        Dim explosion As New Animation()
        explosion.Initialize(explosionTexture, position, 134, 134, 12, 45, Color.White, 1.0F, False)
        explosions.Add(explosion)
    End Sub

    Private Sub UpdateExplosions(ByVal totalTime As TimeSpan, ByVal elapsedTime As TimeSpan)
        For i = explosions.Count - 1 To 0 Step -1
            explosions(i).Update(totalTime, elapsedTime)
            If explosions(i).Active = False Then
                explosions.RemoveAt(i)
            End If
        Next i
    End Sub

#End Region


#Region "Manage Enemies"

    Private Sub AddEnemy()
        ' Initialize the animation object.
        Dim enemyAnimation As New Animation()
        enemyAnimation.Initialize(enemyTexture, Vector2.Zero, 65, 67, 8, 30, Color.White, 1.0F, True)

        ' Randomly generate the position of the enemy, and add the new enemy to the list.
        Dim position As New Vector2(GraphicsDevice.Viewport.Width + enemyTexture.Width \ 2, random.Next(100, GraphicsDevice.Viewport.Height - 100))
        Dim enemy As New Enemy()
        enemy.Initialize(enemyAnimation, position)
        enemies.Add(enemy)
    End Sub

    Private Sub UpdateEnemies(ByVal totalTime As TimeSpan, ByVal elapsedTime As TimeSpan)
        ' Spawn a new enemy every 1.5 seconds.
        If totalTime.Subtract(previousSpawnTime) > enemySpawnTime Then
            previousSpawnTime = totalTime
            AddEnemy()
        End If

        ' Update the enemies.
        For i = enemies.Count - 1 To 0 Step -1
            enemies(i).Update(totalTime, elapsedTime)
            If enemies(i).Active = False Then
                If enemies(i).Health <= 0 Then
                    ' Add an explosion, with sound, and increment the score.
                    AddExplosion(enemies(i).Position)
                    collisionSound.Play()
                    score += enemies(i).Value
                End If
                enemies.RemoveAt(i)
            End If
        Next i
    End Sub

#End Region


#Region "Manage Projectiles"

    Private Sub AddProjectile(ByVal position As Vector2)
        Dim projectile As New Projectile()
        projectile.Initialize(GraphicsDevice.Viewport, projectileTexture, position)
        projectiles.Add(projectile)
    End Sub

    Private Sub UpdateProjectiles()
        For i = projectiles.Count - 1 To 0 Step -1
            projectiles(i).Update()

            If projectiles(i).Active = False Then
                projectiles.RemoveAt(i)
            End If
        Next i
    End Sub

#End Region


#Region "Navigation Events"

    Protected Overrides Sub OnNavigatedTo(ByVal e As NavigationEventArgs)
        CType(App.Current, MangoClient.App).InitializeXnaApplication()

        ' Set the sharing mode of the graphics device to turn on XNA rendering
        SharedGraphicsDeviceManager.Current.GraphicsDevice.SetSharingMode(True)

        ' Always load content, to allow for the case where the user uses a Launcher 
        ' (eg, the help/support link which fires up the browser) before selecting
        ' this game page: that will come through as a StartState.TombstoneResurrected.
        If Not isContentLoaded Then
            LoadContent()
            InitializePlayer(initialPlayerPosition)
        End If

        Dim tmpScore As Integer
        Dim tmpHealth As Integer
        Dim tmpPosition As Vector2
        Select Case App.StartState
            Case StartState.FreshStart
                ' FreshStart - no state to retrieve.
            Case StartState.TombstoneResurrected
                ' Resurrected - we can retrieve state.
                If IsolatedStorageSettings.ApplicationSettings.TryGetValue(Of Integer)("score", tmpScore) Then
                    score = tmpScore
                End If
                If IsolatedStorageSettings.ApplicationSettings.TryGetValue(Of Integer)("health", tmpHealth) Then
                    player.Health = tmpHealth
                End If
                If IsolatedStorageSettings.ApplicationSettings.TryGetValue(Of Vector2)("position", tmpPosition) Then
                    player.Position = tmpPosition
                    InitializePlayer(tmpPosition)
                End If

            Case StartState.NonTombstoneReactivated
                ' Reactivated - no need to retrieve state.
        End Select

        timer.Start()
    End Sub

    Protected Overrides Sub OnNavigatedFrom(ByVal e As NavigationEventArgs)
        ' Make sure we stop the warning sound. This is for the case where
        ' the warning sound is playing when the user hits the Back key.
        If warningSoundInstance.State = SoundState.Playing Then
            warningSoundInstance.Stop()
        End If

        ' Stop the timer
        timer.Stop()

        ' Set the sharing mode of the graphics device to turn off XNA rendering
        SharedGraphicsDeviceManager.Current.GraphicsDevice.SetSharingMode(False)

        ' Persist settings between the game and the main application.
        IsolatedStorageSettings.ApplicationSettings("score") = score
        IsolatedStorageSettings.ApplicationSettings("health") = player.Health
        IsolatedStorageSettings.ApplicationSettings("position") = player.Position
    End Sub

#End Region


#Region "Update"

    Private Sub OnUpdate(ByVal sender As Object, ByVal e As GameTimerEventArgs)
        ' Update the parallaxing background even when the game is over.
        backgroundLayer1.Update()
        backgroundLayer2.Update()

        ' Update the player, enemies, collision, projectiles and explosions
        ' only when the player is still alive.
        If isPlayerAlive Then
            UpdatePlayer(e.TotalTime, e.ElapsedTime)
            UpdateEnemies(e.TotalTime, e.ElapsedTime)
            UpdateCollision()
            UpdateProjectiles()
            UpdateExplosions(e.TotalTime, e.ElapsedTime)
        End If
    End Sub

    Private Sub UpdatePlayer(ByVal totalTime As TimeSpan, ByVal elapsedTime As TimeSpan)
        player.Update(totalTime, elapsedTime)

        ' Windows Phone Controls.
        Do While TouchPanel.IsGestureAvailable
            Dim gesture As GestureSample = TouchPanel.ReadGesture()
            If gesture.GestureType = GestureType.FreeDrag Then
                player.Position += gesture.Delta
            End If
        Loop

        ' Make sure that the player does not go out of bounds.
        player.Position.X = MathHelper.Clamp(player.Position.X, xBounds, GraphicsDevice.Viewport.Width - xBounds)
        player.Position.Y = MathHelper.Clamp(player.Position.Y, yBounds, GraphicsDevice.Viewport.Height - yBounds)

        ' Fire only every interval we set as the fireTime.
        If totalTime.Subtract(previousFireTime) > fireTime Then
            ' Reset our current time, and add the projectile to the front/center of the player:
            previousFireTime = totalTime
            AddProjectile(player.Position + New Vector2(player.Width \ 2, 0))
            projectileSound.Play()
        End If

        ' Warning: health approaching zero.
        If player.Health <= 10 AndAlso warningSoundInstance.State <> SoundState.Playing Then
            warningSoundInstance.Play()
        End If

        ' Game over.
        If player.Health <= 0 Then
            warningSoundInstance.Stop()
            gameOverSound.Play()
            isPlayerAlive = False
        End If
    End Sub

    Private Sub UpdateCollision()
        ' Use the Rectangle's built-in intersect function to 
        ' determine if two objects are overlapping.
        Dim rectangle1 As Rectangle
        Dim rectangle2 As Rectangle

        ' Only create the rectangle once for the player.
        rectangle1 = New Rectangle(CInt(Fix(player.Position.X)), CInt(Fix(player.Position.Y)), player.Width, player.Height)

        ' Do the collision between the player and the enemies.
        For i = 0 To enemies.Count - 1
            rectangle2 = New Rectangle(CInt(Fix(enemies(i).Position.X)), CInt(Fix(enemies(i).Position.Y)), enemies(i).Width, enemies(i).Height)

            ' Determine if the two objects collided with each other.
            If rectangle1.Intersects(rectangle2) Then
                ' Subtract the health from the player based on the enemy damage.
                player.Health -= enemies(i).Damage

                ' Since the enemy collided with the player destroy it.
                enemies(i).Health = 0

                ' If the player health is less than zero we died.
                If player.Health <= 0 Then
                    player.Active = False
                End If
            End If
        Next i

        ' Projectile vs Enemy Collision.
        For i = 0 To projectiles.Count - 1
            For j = 0 To enemies.Count - 1
                ' Create the rectangles we need to determine if we collided with each other.
                rectangle1 = New Rectangle(CInt(Fix(projectiles(i).Position.X)) - projectiles(i).Width \ 2, CInt(Fix(projectiles(i).Position.Y)) - projectiles(i).Height \ 2, projectiles(i).Width, projectiles(i).Height)

                rectangle2 = New Rectangle(CInt(Fix(enemies(j).Position.X)) - enemies(j).Width \ 2, CInt(Fix(enemies(j).Position.Y)) - enemies(j).Height \ 2, enemies(j).Width, enemies(j).Height)

                ' Determine if the two objects collided with each other.
                If rectangle1.Intersects(rectangle2) Then
                    enemies(j).Health -= projectiles(i).Damage
                    projectiles(i).Active = False
                End If
            Next j
        Next i
    End Sub

#End Region


#Region "Draw"

    Private Sub OnDraw(ByVal sender As Object, ByVal e As GameTimerEventArgs)
        GraphicsDevice.Clear(Color.Black)
        spriteBatch.Begin()
        DrawBackground()

        If isPlayerAlive Then
            ' Draw the player, enemies, projectiles and explosions.
            player.Draw(spriteBatch)
            For i = 0 To enemies.Count - 1
                enemies(i).Draw(spriteBatch)
            Next i
            For i = 0 To projectiles.Count - 1
                projectiles(i).Draw(spriteBatch)
            Next i
            For i = 0 To explosions.Count - 1
                explosions(i).Draw(spriteBatch)
            Next i

            ' Draw the score and player health.
            spriteBatch.DrawString(font, "mangotini score:  " & score, New Vector2(GraphicsDevice.Viewport.TitleSafeArea.X + 12, GraphicsDevice.Viewport.TitleSafeArea.Y), Color.Orange)
            spriteBatch.DrawString(font, "field strength:  " & player.Health, New Vector2(GraphicsDevice.Viewport.TitleSafeArea.X + 12, GraphicsDevice.Viewport.TitleSafeArea.Y + 36), Color.Orange)
        Else
            spriteBatch.DrawString(font, "game over!", New Vector2(GraphicsDevice.Viewport.X + 150, GraphicsDevice.Viewport.Y + 340), Color.Orange)
            spriteBatch.DrawString(font, "score:  " & score, New Vector2(GraphicsDevice.Viewport.X + 150, GraphicsDevice.Viewport.Y + 376), Color.Orange)
        End If

        spriteBatch.End()
    End Sub

    Private Sub DrawBackground()
        spriteBatch.Draw(mainBackground, Vector2.Zero, Color.White)
        backgroundLayer1.Draw(spriteBatch)
        backgroundLayer2.Draw(spriteBatch)
    End Sub

#End Region

End Class