﻿Imports System.Windows.Media.Imaging
Imports System.Windows.Media


Partial Public Class RecipePage
    Inherits PhoneApplicationPage

#Region "Fields"

    Private recipeID As Integer
    Private thisRecipe As Recipe
    Private thisPageUri As String
    Private tile As ShellTile

#End Region


#Region "Init"

    Public Sub New()
        InitializeComponent()
    End Sub

    Private Function GetPinImageForTheme(ByVal pinState As PinState) As Uri
        Dim pinUri As Uri = Nothing
        Dim dark As Visibility = CType(Resources("PhoneDarkThemeVisibility"), Visibility)
        Dim uriString As String = "Images/Pins/"
        If dark = Visibility.Visible Then
            uriString &= If((pinState = pinState.Pin), "Pin_white.png", "Unpin_white.png")
            pinUri = New Uri(uriString, UriKind.Relative)
        End If

        uriString &= If((pinState = pinState.Pin), "Pin_black.png", "Unpin_black.png")
        pinUri = New Uri(uriString, UriKind.Relative)
        Return pinUri
    End Function

#End Region


#Region "OnNavigatedTo"

    Protected Overrides Sub OnNavigatedTo(ByVal e As NavigationEventArgs)
        recipeID = Integer.Parse(NavigationContext.QueryString("ID"))

        ' Get the data for this Recipe.
        App.ViewModel.LoadRecipes()
        For Each r As Recipe In App.ViewModel.Recipes
            If r.ID = recipeID Then
                thisRecipe = r
                DataContext = thisRecipe
                Exit For
            End If
        Next r

        ' Check if the user has pinned this recipe tile to the Start page.
        thisPageUri = String.Format("/RecipePage.xaml?ID={0}", recipeID)
        tile = ShellTile.ActiveTiles.FirstOrDefault(Function(x) x.NavigationUri.ToString().Contains(thisPageUri))

        Dim pinBrush As ImageBrush = CType(PinUnpin.Background, ImageBrush)
        If tile IsNot Nothing Then
            pinBrush.ImageSource = New BitmapImage(GetPinImageForTheme(PinState.Unpin))
        Else
            pinBrush.ImageSource = New BitmapImage(GetPinImageForTheme(PinState.Pin))
        End If
        PinUnpin.Background = pinBrush
    End Sub

#End Region


#Region "PinUnpin_Click"

    Private Sub PinUnpin_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
        ' If this tile doesn't already exist, create it.
        Dim pinBrush As ImageBrush = CType(PinUnpin.Background, ImageBrush)
        tile = ShellTile.ActiveTiles.FirstOrDefault(Function(x) x.NavigationUri.ToString().Contains(thisPageUri))
        If tile Is Nothing Then
            ' Set up the new tile based on the current Recipe data.
            Dim tileData As StandardTileData = New StandardTileData With {.BackgroundImage = New Uri(thisRecipe.Photo, UriKind.RelativeOrAbsolute), .Title = thisRecipe.Name, .BackTitle = "Lovely Mangoes!", .BackBackgroundImage = New Uri("Images/BackTile.png", UriKind.Relative)}

            ' Toggle the Pin/Unpin button to the 'Unpin' image.
            pinBrush.ImageSource = New BitmapImage(GetPinImageForTheme(PinState.Unpin))

            ' Create the tile and pin to start. Note this will deactivate the app.
            ShellTile.Create(New Uri(thisPageUri, UriKind.Relative), tileData)
        Else
            ' This tile already exists, so the user must have asked us to unpin it.
            tile.Delete()
            pinBrush.ImageSource = New BitmapImage(GetPinImageForTheme(PinState.Pin))
        End If
        PinUnpin.Background = pinBrush
    End Sub

#End Region

End Class

Public Enum PinState
    Pin
    Unpin
End Enum
