﻿Imports System.Collections.Generic


Public Class MainViewModel

#Region "Fields"

    Private mangoDb As MangoDataContext

#End Region


#Region "Properties"

    Private _isDataLoaded As Boolean
    Public ReadOnly Property IsDataLoaded() As Boolean
        Get
            Return _isDataLoaded
        End Get
    End Property

    Private _season As String
    Public ReadOnly Property Season() As String
        Get
            Return _season
        End Get
    End Property

    Private _categories As List(Of Category)
    Public ReadOnly Property Categories() As List(Of Category)
        Get
            Return _categories
        End Get
    End Property

    Private _recipes As List(Of Recipe)
    Public ReadOnly Property Recipes() As List(Of Recipe)
        Get
            Return _recipes
        End Get
    End Property

    Private _facts As List(Of Fact)
    Public ReadOnly Property Facts() As List(Of Fact)
        Get
            Return _facts
        End Get
    End Property

    Private _cocktails As List(Of Cocktail)
    Public ReadOnly Property Cocktails() As List(Of Cocktail)
        Get
            Return _cocktails
        End Get
    End Property

    Private _seasonalHighlights As List(Of SeasonalHighlight)
    Public ReadOnly Property SeasonalHighlights() As List(Of SeasonalHighlight)
        Get
            Return _seasonalHighlights
        End Get
    End Property

#End Region


#Region "Init"

    Public Sub New()
        ' Find out what season we're in.
        _season = String.Empty
        Dim currentMonth As Integer = Date.Now.Month
        If currentMonth >= 3 AndAlso currentMonth <= 5 Then
            _season = "spring"
        ElseIf currentMonth >= 6 AndAlso currentMonth <= 8 Then
            _season = "summer"
        ElseIf currentMonth >= 9 AndAlso currentMonth <= 11 Then
            _season = "autumn"
        ElseIf currentMonth = 12 OrElse currentMonth = 1 OrElse currentMonth = 2 Then
            _season = "winter"
        End If
    End Sub

#End Region


#Region "LoadData"

    Friend Sub LoadData()
        mangoDb = New MangoDataContext()
        If Not mangoDb.DatabaseExists() Then
            mangoDb.CreateDatabase()
        End If

        _categories = New List(Of Category)()
        _categories.Add(New Category With {.ID = 1, .Name = "recipes"})
        _categories.Add(New Category With {.ID = 2, .Name = "fun facts"})
        _categories.Add(New Category With {.ID = 3, .Name = "cocktail bar"})
        _categories.Add(New Category With {.ID = 4, .Name = "mango, the game"})

        ' Delay-load the Recipes, Facts and Cocktails, to speed up app startup.IQueryable(Of Recipe) recipesQuery = From r As Recipe In mangoDb.Recipes
        ' Select r
        ' recipes = New List(Of Recipe)(recipesQuery)

        ' Dim factsQuery As IQueryable(Of Fact) = From f As Fact In mangoDb.Facts
        ' Select f
        ' Facts = New List(Of Fact)(factsQuery)
        ' Dim cocktailsQuery As IQueryable(Of Cocktail) = From t As Cocktail In mangoDb.Cocktails
        ' Select t
        ' Cocktails = New List(Of Cocktail)(cocktailsQuery)

        ' Dim seasonalRecipes = From r In Recipes
        ' Where(r.Season = Season)
        ' Select New With {Key r.ID, Key r.Name, Key r.Photo}
        ' Dim seasonalCocktails = From c In Cocktails
        ' Where(c.Season = Season)
        ' Select New With {Key c.ID, Key c.Name, Key c.Photo}


        Dim seasonalRecipes = From r In mangoDb.Recipes
                              Where r.Season = Season
                              Select New With {Key r.ID, Key r.Name, Key r.Photo}
        Dim seasonalCocktails = From c In mangoDb.Cocktails
                                Where c.Season = Season
                                Select New With {Key c.ID, Key c.Name, Key c.Photo}

        _seasonalHighlights = New List(Of SeasonalHighlight)()
        For Each v In seasonalRecipes
            _seasonalHighlights.Add(New SeasonalHighlight With {.ID = v.ID, .Name = v.Name, .Photo = v.Photo, .SourceTable = "Recipes"})
        Next v
        For Each v In seasonalCocktails
            _seasonalHighlights.Add(New SeasonalHighlight With {.ID = v.ID, .Name = v.Name, .Photo = v.Photo, .SourceTable = "Cocktails"})
        Next v

        _isDataLoaded = True
    End Sub

    Friend Sub LoadFacts()
        If _facts Is Nothing OrElse _facts.Count = 0 Then
            Dim factsQuery As IQueryable(Of Fact) = From f As Fact In mangoDb.Facts
                                                    Select f
            _facts = New List(Of Fact)(factsQuery)
        End If
    End Sub

    Friend Sub LoadRecipes()
        If _recipes Is Nothing OrElse _recipes.Count = 0 Then
            Dim recipesQuery As IQueryable(Of Recipe) = From r As Recipe In mangoDb.Recipes
                                                        Select r
            _recipes = New List(Of Recipe)(recipesQuery)
        End If
    End Sub

    Friend Sub LoadCocktails()
        If _cocktails Is Nothing OrElse _cocktails.Count = 0 Then
            Dim cocktailsQuery As IQueryable(Of Cocktail) = From t As Cocktail In mangoDb.Cocktails
                                                            Select t
            _cocktails = New List(Of Cocktail)(cocktailsQuery)
        End If
    End Sub

#End Region

End Class