﻿Imports System.Data.Linq
Imports System.Data.Linq.Mapping
Imports System.ComponentModel

Public Class MangoDataContext
    Inherits DataContext
    ' This DB is readonly, so we can access it directly from the install folder.
    Public Sub New()
        MyBase.New("Data Source=appdata:/Mangolicious.sdf;File Mode=read only;")
    End Sub

    Public Recipes As Table(Of Recipe)
    Public Facts As Table(Of Fact)
    Public Cocktails As Table(Of Cocktail)
    Public SeasonalHighlights As Table(Of SeasonalHighlight)
End Class


#Region "Recipes"

<Table()>
Public Class Recipe
    Private _id As Integer
    <Column(IsPrimaryKey:=True, IsDbGenerated:=True, DbType:="INT NOT NULL Identity", CanBeNull:=False, AutoSync:=AutoSync.OnInsert)>
    Public Property ID() As Integer
        Get
            Return _id
        End Get
        Set(ByVal value As Integer)
            If _id <> value Then
                _id = value
            End If
        End Set
    End Property

    Private _name As String
    <Column(DbType:="NVARCHAR(32)")>
    Public Property Name() As String
        Get
            Return _name
        End Get
        Set(ByVal value As String)
            If _name <> value Then
                _name = value
            End If
        End Set
    End Property

    Private _photo As String
    <Column(DbType:="NVARCHAR(255)")>
    Public Property Photo() As String
        Get
            Return _photo
        End Get
        Set(ByVal value As String)
            If _photo <> value Then
                _photo = value
            End If
        End Set
    End Property

    Private _ingredients As String
    <Column(DbType:="NVARCHAR(512)")>
    Public Property Ingredients() As String
        Get
            Return _ingredients
        End Get
        Set(ByVal value As String)
            If _ingredients <> value Then
                _ingredients = value
            End If
        End Set
    End Property

    Private _instructions As String
    <Column(DbType:="NVARCHAR(1024)")>
    Public Property Instructions() As String
        Get
            Return _instructions
        End Get
        Set(ByVal value As String)
            If _instructions <> value Then
                _instructions = value
            End If
        End Set
    End Property

    Private _season As String
    <Column(DbType:="NVARCHAR(8)")>
    Public Property Season() As String
        Get
            Return _season
        End Get
        Set(ByVal value As String)
            If _season <> value Then
                _season = value
            End If
        End Set
    End Property
End Class

#End Region


#Region "Facts"

<Table()>
Public Class Fact
    Private _id As Integer
    <Column(IsPrimaryKey:=True, IsDbGenerated:=True, DbType:="INT NOT NULL Identity", CanBeNull:=False, AutoSync:=AutoSync.OnInsert)>
    Public Property ID() As Integer
        Get
            Return _id
        End Get
        Set(ByVal value As Integer)
            If _id <> value Then
                _id = value
            End If
        End Set
    End Property

    Private _name As String
    <Column(DbType:="NVARCHAR(32)")>
    Public Property Name() As String
        Get
            Return _name
        End Get
        Set(ByVal value As String)
            If _name <> value Then
                _name = value
            End If
        End Set
    End Property

    Private _photo As String
    <Column(DbType:="NVARCHAR(255)")>
    Public Property Photo() As String
        Get
            Return _photo
        End Get
        Set(ByVal value As String)
            If _photo <> value Then
                _photo = value
            End If
        End Set
    End Property

    Private _body As String
    <Column(DbType:="NVARCHAR(1024)")>
    Public Property Body() As String
        Get
            Return _body
        End Get
        Set(ByVal value As String)
            If _body <> value Then
                _body = value
            End If
        End Set
    End Property
End Class

#End Region


#Region "Cocktails"

<Table()>
Public Class Cocktail
    Private _id As Integer
    <Column(IsPrimaryKey:=True, IsDbGenerated:=True, DbType:="INT NOT NULL Identity", CanBeNull:=False, AutoSync:=AutoSync.OnInsert)>
    Public Property ID() As Integer
        Get
            Return _id
        End Get
        Set(ByVal value As Integer)
            If _id <> value Then
                _id = value
            End If
        End Set
    End Property

    Private _name As String
    <Column(DbType:="NVARCHAR(32)")>
    Public Property Name() As String
        Get
            Return _name
        End Get
        Set(ByVal value As String)
            If _name <> value Then
                _name = value
            End If
        End Set
    End Property

    Private _photo As String
    <Column(DbType:="NVARCHAR(255)")>
    Public Property Photo() As String
        Get
            Return _photo
        End Get
        Set(ByVal value As String)
            If _photo <> value Then
                _photo = value
            End If
        End Set
    End Property

    Private _ingredients As String
    <Column(DbType:="NVARCHAR(512)")>
    Public Property Ingredients() As String
        Get
            Return _ingredients
        End Get
        Set(ByVal value As String)
            If _ingredients <> value Then
                _ingredients = value
            End If
        End Set
    End Property

    Private _instructions As String
    <Column(DbType:="NVARCHAR(1024)")>
    Public Property Instructions() As String
        Get
            Return _instructions
        End Get
        Set(ByVal value As String)
            If _instructions <> value Then
                _instructions = value
            End If
        End Set
    End Property

    Private _season As String
    <Column(DbType:="NVARCHAR(8)")>
    Public Property Season() As String
        Get
            Return _season
        End Get
        Set(ByVal value As String)
            If _season <> value Then
                _season = value
            End If
        End Set
    End Property
End Class

#End Region


#Region "SeasonalHighlights"

' NOTE SeasonalHighlights is the one table in the DataContext that does not 
' have a matching table in the SQL-CE DB. The data for this table is composed
' at runtime from the SQL Cocktails and Recipes tables (SQL-CE doesn't support views).

<Table()>
Public Class SeasonalHighlight
    ' Unlike the ID column in other tables, the ID column in the
    ' SeasonalHighlights table is not an Identity column, nor a key,
    ' it can be null, and it is not auto-generated.
    Private _id As Integer
    <Column(IsPrimaryKey:=False, IsDbGenerated:=False, DbType:="INT NOT NULL", CanBeNull:=True, AutoSync:=AutoSync.OnInsert)>
    Public Property ID() As Integer
        Get
            Return _id
        End Get
        Set(ByVal value As Integer)
            If _id <> value Then
                _id = value
            End If
        End Set
    End Property

    Private _sourceTable As String
    <Column(DbType:="NVARCHAR(16)")>
    Public Property SourceTable() As String
        Get
            Return _sourceTable
        End Get
        Set(ByVal value As String)
            If _sourceTable <> value Then
                _sourceTable = value
            End If
        End Set
    End Property

    Private _name As String
    <Column(DbType:="NVARCHAR(32)")>
    Public Property Name() As String
        Get
            Return _name
        End Get
        Set(ByVal value As String)
            If _name <> value Then
                _name = value
            End If
        End Set
    End Property

    Private _photo As String
    <Column(DbType:="NVARCHAR(255)")>
    Public Property Photo() As String
        Get
            Return _photo
        End Get
        Set(ByVal value As String)
            If _photo <> value Then
                _photo = value
            End If
        End Set
    End Property

    Private _season As String
    <Column(DbType:="NVARCHAR(8)")>
    Public Property Season() As String
        Get
            Return _season
        End Get
        Set(ByVal value As String)
            If _season <> value Then
                _season = value
            End If
        End Set
    End Property
End Class

#End Region


#Region "Categories"

' Category is a simple type, there is no corresponding
' table in the database. 
Public Class Category
    Private _id As Integer
    Public Property ID() As Integer
        Get
            Return _id
        End Get
        Set(ByVal value As Integer)
            If _id <> value Then
                _id = value
            End If
        End Set
    End Property

    Private _name As String
    Public Property Name() As String
        Get
            Return _name
        End Get
        Set(ByVal value As String)
            If _name <> value Then
                _name = value
            End If
        End Set
    End Property
End Class

#End Region
